
//   Copyright 2020 Esri
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

package com.example.app;

import java.util.List;
import java.util.concurrent.ExecutionException;

import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.paint.Color;
import javafx.scene.control.Alert;
import javafx.scene.control.TextField;

import com.esri.arcgisruntime.concurrent.ListenableFuture;
import com.esri.arcgisruntime.mapping.view.Graphic;
import com.esri.arcgisruntime.mapping.view.GraphicsOverlay;
import com.esri.arcgisruntime.symbology.SimpleMarkerSymbol;
import com.esri.arcgisruntime.symbology.TextSymbol;
import com.esri.arcgisruntime.tasks.geocode.GeocodeParameters;
import com.esri.arcgisruntime.tasks.geocode.GeocodeResult;
import com.esri.arcgisruntime.tasks.geocode.LocatorTask;

import com.esri.arcgisruntime.ArcGISRuntimeEnvironment;
import com.esri.arcgisruntime.mapping.ArcGISMap;
import com.esri.arcgisruntime.mapping.BasemapStyle;
import com.esri.arcgisruntime.mapping.Viewpoint;
import com.esri.arcgisruntime.mapping.view.MapView;

import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.layout.StackPane;
import javafx.stage.Stage;

public class App extends Application {

  private MapView mapView;

  private GeocodeParameters geocodeParameters;
  private GraphicsOverlay graphicsOverlay;
  private LocatorTask locatorTask;

  private TextField searchBox;

  public static void main(String[] args) {
    Application.launch(args);
  }


  @Override
  public void start(Stage stage) {

    // set the title and size of the stage and show it
    stage.setTitle("Search for an address tutorial");
    stage.setWidth(800);
    stage.setHeight(700);
    stage.show();
    // create a JavaFX scene with a stack pane as the root node
    // and add it to the scene

    StackPane stackPane = new StackPane();
    Scene scene = new Scene(stackPane);
    stage.setScene(scene);

    // Note: it is not best practice to store API keys in source code.
    // The API key is referenced here for the convenience of this tutorial.
    String yourApiKey = "YOUR_API_KEY";
    ArcGISRuntimeEnvironment.setApiKey(yourApiKey);

    // create a map view to display the map and add it to the stack pane
    mapView = new MapView();
    stackPane.getChildren().add(mapView);

    ArcGISMap map = new ArcGISMap(BasemapStyle.ARCGIS_TOPOGRAPHIC);
    // set the map on the map view
    mapView.setMap(map);
    mapView.setViewpoint(new Viewpoint(34.02700, -118.80543, 144447.638572));

    // create a graphics overlay and add it to the map view
    graphicsOverlay = new GraphicsOverlay();

    mapView.getGraphicsOverlays().add(graphicsOverlay);

    setupTextField();

    createLocatorTaskAndDefaultParameters();

    searchBox.setOnAction(event -> {
      String address = searchBox.getText();
      if (!address.isBlank()) {
        performGeocode(address);
      }
    });

    stackPane.getChildren().add(searchBox);
    StackPane.setAlignment(searchBox, Pos.TOP_LEFT);
    StackPane.setMargin(searchBox, new Insets(10, 0, 0, 10));

  }

  private void setupTextField() {
    searchBox = new TextField();
    searchBox.setMaxWidth(400);
    searchBox.setPromptText("Search for an address");
  }

  private void createLocatorTaskAndDefaultParameters() {
    locatorTask = new LocatorTask("https://geocode-api.arcgis.com/arcgis/rest/services/World/GeocodeServer");

    geocodeParameters = new GeocodeParameters();
    geocodeParameters.getResultAttributeNames().add("*");
    geocodeParameters.setMaxResults(1);
    geocodeParameters.setOutputSpatialReference(mapView.getSpatialReference());
  }

  private void performGeocode(String address) {
    ListenableFuture<List<GeocodeResult>> geocodeResults = locatorTask.geocodeAsync(address, geocodeParameters);

    geocodeResults.addDoneListener(() -> {
      try {
        List<GeocodeResult> geocodes = geocodeResults.get();
        if (geocodes.size() > 0) {
          GeocodeResult result = geocodes.get(0);

          displayResult(result);

        } else {
          new Alert(Alert.AlertType.INFORMATION, "No results found.").show();
        }
      } catch (InterruptedException | ExecutionException e) {
        new Alert(Alert.AlertType.ERROR, "Error getting result.").show();
        e.printStackTrace();
      }
    });
  }

  private void displayResult(GeocodeResult geocodeResult) {
    graphicsOverlay.getGraphics().clear(); // clears the overlay of any previous result

    // create a graphic to display the address text
    String label = geocodeResult.getLabel();
    TextSymbol textSymbol = new TextSymbol(18, label, Color.BLACK, TextSymbol.HorizontalAlignment.CENTER, TextSymbol.VerticalAlignment.BOTTOM);
    Graphic textGraphic = new Graphic(geocodeResult.getDisplayLocation(), textSymbol);
    graphicsOverlay.getGraphics().add(textGraphic);

    // create a graphic to display the location as a red square
    SimpleMarkerSymbol markerSymbol = new SimpleMarkerSymbol(SimpleMarkerSymbol.Style.SQUARE, Color.RED, 12.0f);
    Graphic markerGraphic = new Graphic(geocodeResult.getDisplayLocation(), geocodeResult.getAttributes(), markerSymbol);
    graphicsOverlay.getGraphics().add(markerGraphic);

    mapView.setViewpointCenterAsync(geocodeResult.getDisplayLocation());
  }

  /**
   * Stops and releases all resources used in application.
   */
  @Override
  public void stop() {
    if (mapView != null) {
      mapView.dispose();
    }
  }

}


